library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library unisim;
use unisim.vcomponents.all;

entity ps2kbd is
    Port ( reset: in boolean;
	 		  clk: in std_logic;
           kb_clk_src: in std_logic;
			  alt_keypad_mode: in boolean;
           read: in std_logic;
           data: out std_logic_vector(7 downto 0);
           data_avail: out std_logic;
           kb_clk: inout std_logic;
           kb_data: inout std_logic);
end ps2kbd;

architecture RTL of ps2kbd is

	component PosEdge
		port
		  (reset: in boolean;
			clk: in std_logic;
			inp: in std_logic;          
			outp: out std_logic);
	end component;

  	attribute INIT_00: string;  attribute INIT_01: string; attribute INIT_02: string;  attribute INIT_03: string;
  	attribute INIT_04: string;  attribute INIT_05: string; attribute INIT_06: string;  attribute INIT_07: string;
  	attribute INIT_08: string;  attribute INIT_09: string; attribute INIT_0A: string;  attribute INIT_0B: string;
  	attribute INIT_0C: string;  attribute INIT_0D: string; attribute INIT_0E: string;  attribute INIT_0F: string;

	-- BASE TABLE
  	attribute INIT_00 of rom : label is "E0327761737AE0E0E03171E1E0E2E0E0E06009E0E0E0E0E0E0D1D0D2E0E0E0E0";
  	attribute INIT_01 of rom : label is "E03837756A6DE0E0E036796768626EE0E0E07274667620E0E0333465647863E0";
  	attribute INIT_02 of rom : label is "E0E05CE0E00DE1E4E0E03D5BE027E0E0E02D703B6C2F2EE0E039306F696BE0E0";
  	attribute INIT_03 of rom : label is "E0E0998A8D938BE0E01E989695928E90E0E0E09794E091E0E008E0E0E0E0E0E0";
	-- CTRL TABLE
	attribute INIT_04 of rom : label is "E0001701131AE0E0E0E011E1E0E2E0E0E01E09E0E0E0E0E0E0D1D0D2E0E0E0E0";
	attribute INIT_05 of rom : label is "E0E0E0150A0DE0E0E01E190708020EE0E0E01214061620E0E0E0E005041803E0";
	attribute INIT_06 of rom : label is "E0E01CE01D0DE1E4E0E01D1BE0E0E0E0E0E010E00CE0E0E0E0E0E00F090BE0E0";
	attribute INIT_07 of rom : label is "E0E0998A8D938BE0E01E989695928E90E0E0E09794E091E0E008E0E0E0E0E0E0";
	-- SHIFT TABLE
  	attribute INIT_08 of rom : label is "E0605741535AE0E0E02151E1E0E2E0E0E07E09E0E0E0E0E0E0D1D0D2E0E0E0E0";
  	attribute INIT_09 of rom : label is "E0E0E0150A0DE0E0E01E190708020EE0E0E01214061620E0E0E0E005041803E0";
  	attribute INIT_0A of rom : label is "E0E07CE07D0DE1E4E0E02B7BE022E0E0E05F503A4C3F3EE0E028294F494BE0E0";
  	attribute INIT_0B of rom : label is "E0E0E08A8DE08BE0E01EC1C3E0C2E0E0E0E0E0E0C4E0E0E0E008E0E0E0E0E0E0";
	-- EXT TABLE
  	attribute INIT_0C of rom : label is "E0E0E0E0E0E0E0E0E0E0E0E1E0E2E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0";
  	attribute INIT_0D of rom : label is "E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0";
  	attribute INIT_0E of rom : label is "E0E0E0E0E08DE1E4E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0";
  	attribute INIT_0F of rom : label is "E0E0E0E0E0E0E0E0E0E0C1C3E0C2E0E0E0E0E0E0C4E0E0E0E0E0E0E0E0E0E0E0";

	-- low-level receive
  	signal rbit: integer range 0 to 10;
  	signal sr: std_logic_vector(8 downto 0);
  	signal kb_clk_pe, parity: std_logic;
  	signal RC_1, RC_2: std_logic;

	-- ASCII conversion state machine
	signal ctrl, shift, caps: std_logic;

	signal lookup: std_logic_vector(8 downto 0);
	signal lookup_res: std_logic_vector(7 downto 0);
	signal xlate: unsigned(7 downto 0);
	signal lookup_en: std_logic;

	type KStates is
		(Idle,
		 GotExt,
		 GotBrk,
		 GotExtBrk);
	signal state: KStates;

	type CStates is
		(Idle,
		 Alpha0,		-- simple ASCII
		 ESC1,		-- ESC c+40
		 Alpha1,
		 ESC2,		-- ESC c+20
		 Alpha2,
		 ESC3,		-- ESC ? c+60
		 QM3,
		 Alpha3);
	signal cstate: CStates;

begin

  	kb_clk <= 'Z';
  	kb_data <= 'Z';

	rom: RAMB4_S8
	-- n.b. this copy is needed for simulation
-- synopsys translate_off
    generic map
      (
      INIT_00 => X"E0327761737AE0E0E03161E1E0E2E0E0E06009E0E0E0E0E0E0D1D0D2E0E0E0E0",
      INIT_01 => X"E0E03837756A6DE0E0E036796768626EE0E0E07274667620E0E03334656463E0",
      INIT_02 => X"E0E05CE0E00DE1E4E0E03D5BE027E0E0E02D703B6C2F2EE0E039306F696BE0E0",
      INIT_03 => X"E0E0998A8D938BE0E01E989695928E90E0E0E09794E091E0E008E0E0E0E0E0E0",
      INIT_04 => X"E0001701131AE0E0E0E011E1E0E2E0E0E01E09E0E0E0E0E0E0D1D0D2E0E0E0E0",
      INIT_05 => X"E0E0E0150A0DE0E0E01E190708020EE0E0E01214061620E0E0E0E005041803E0",
      INIT_06 => X"E0E01CE01D0DE1E4E0E01D1BE0E0E0E0E0E010E00CE0E0E0E0E0E00F090BE0E0",
      INIT_07 => X"E0E0998A8D938BE0E01E989695928E90E0E0E09794E091E0E008E0E0E0E0E0E0",
      INIT_08 => X"E0605741535AE0E0E02151E1E0E2E0E0E07E09E0E0E0E0E0E0D1D0D2E0E0E0E0",
      INIT_09 => X"E0E0E0150A0DE0E0E01E190708020EE0E0E01214061620E0E0E0E005041803E0",
      INIT_0A => X"E0E07CE07D0DE1E4E0E02B7BE022E0E0E05F503A4C3F3EE0E028294F494BE0E0",
      INIT_0B => X"E0E0E08A8DE08BE0E01EC1C3E0C2E0E0E0E0E0E0C4E0E0E0E008E0E0E0E0E0E0",
      INIT_0C => X"E0E0E0E0E0E0E0E0E0E0E0E1E0E2E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0",
      INIT_0D => X"E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0",
      INIT_0E => X"E0E0E0E0E08DE1E4E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0",
      INIT_0F => X"E0E0E0E0E0E0E0E0E0E0C1C3E0C2E0E0E0E0E0E0C4E0E0E0E0E0E0E0E0E0E0E0"
      )
-- synopsys translate_on
  port map
      (WE => '0', EN => lookup_en, RST => '0', CLK => clk,
       ADDR => lookup, DI => "00000000", DO => lookup_res);

	lookup_en <= '1' when (cstate = Idle) else '0';

	lookup <= "11" & sr(6 downto 0) when (state = GotExt) or (state = GotExtBrk) else
				 "01" & sr(6 downto 0) when ctrl = '1' else
				 "10" & sr(6 downto 0) when (shift = '1') or (caps = '1') else
				 "00" & sr(6 downto 0);

	xlate <= unsigned(lookup_res);

	process (cstate, read, xlate)
	begin
		if read = '1' then
			
			case cstate is

				when Idle =>
					data <= (others => '0');

				when ESC1 | ESC2 | ESC3 =>
					data <= "00011011";			-- <ESC> char

				when Alpha0 =>
					data <= std_logic_vector(xlate);

				when Alpha1 =>
					data <= std_logic_vector(xlate + "10000000");

				when Alpha2 =>
					data <=  std_logic_vector(xlate + "1010000");

				when QM3 =>
					data <= "00111111";			-- <?> char

				when Alpha3 =>
					data <=  std_logic_vector(xlate + "11100000");

			end case;

		else
			data <= (others => 'Z');
		end if;
	end process;

	data_avail <= '1' when cstate /= Idle else '0';

	process (reset, clk, RC_1, RC_2)
	begin
		if reset then
			RC_1 <= '0';
			state <= Idle;
			cstate <= Idle;
			ctrl <= '0';
			shift <= '0';
			caps <= '0';
		elsif rising_edge(clk) then

			-- check for end of read strobe from host; if true,
			-- advance read state
			if read = '1' then
				case cstate is
					when ESC1 =>
						cstate <= Alpha1;
					
					when ESC2 =>
						cstate <= Alpha2;
					
					when ESC3 =>
						cstate <= QM3;
					
					when QM3 =>
						cstate <= Alpha3;

					when others =>
						cstate <= Idle;
				end case;
			end if;

			-- check for new code from keyboard
			if RC_1 /= RC_2 then
				RC_1 <= RC_2;

				case sr(7 downto 0) is

					when "11100000" =>	-- ext
						state <= GotExt;

					when "11110000" =>	-- break
						if state = Idle then
							state <= GotBrk;
						else
							state <= GotExtBrk;
						end if;

					when others =>			-- a keypress, use lookup value or modify shift status

						if (state = Idle) or (state = GotExt) then
							if xlate(7) = '0' then
								cstate <= Alpha0;
								state <= Idle;
							elsif xlate(6) = '0' then
								if alt_keypad_mode then
									cstate <= ESC3;
								else
									cstate <= ESC2;
								end if;
								state <= Idle;
							elsif xlate(5) = '0' then
								cstate <= ESC1;
								state <= Idle;
							end if;
						end if;

						if xlate(7 downto 5) = "111" then

							state <= Idle;

							if xlate(0) = '1' then	-- shift
								if state = GotBrk then
									shift <= '0';
								else
									shift <= '1';
								end if;
							end if;

							if xlate(1) = '1' then
								if state = GotBrk then
									ctrl <= '0';
								else
									ctrl <= '1';
								end if;
							end if;

							if xlate(2) = '1' then
								if state /= GotBrk then
									caps <= not caps;
								end if;
							end if;

						end if;

				end case;
			end if;
		end if;
	end process;

	edge_det: PosEdge port map (reset, clk, kb_clk, kb_clk_pe);

	process (reset, clk, kb_clk_pe, rbit, kb_data, RC_1)
	begin
		if reset then
			rbit <= 10;
			RC_2 <= '0';
		elsif rising_edge(clk) then
			if kb_clk_pe = '1' then
				if rbit = 10 then
					-- start reception if 0
					if kb_data = '0' then
						parity <= '0';
						rbit <= 0;
					end if;
				else
					parity <= parity xor kb_data;
					if rbit = 9 then
						if (kb_data = '1') and (parity = '1') then
							RC_2 <= not RC_1;		-- valid data available
						end if;
					else
						sr(7 downto 0) <= sr(8 downto 1);
						sr(8) <= kb_data;
					end if;
					rbit <= rbit + 1;
				end if;
			end if;
		end if;
	end process;


end RTL;
